@@ -18,6 +18,7 @@
 #define SEWENEW_REDISPLUSPLUS_REPLY_H
 
 #include <cassert>
+#include <cstdlib>
 #include <string>
 #include <iterator>
 #include <memory>
@@ -115,7 +116,7 @@ template <typename T, typename std::enable_if<IsAssociativeContainer<T>::value,
 T parse(ParseTag<T>, redisReply &reply);
 
 template <typename Output>
-long long parse_scan_reply(redisReply &reply, Output output);
+Cursor parse_scan_reply(redisReply &reply, Output output);
 
 inline bool is_error(redisReply &reply) {
     return reply.type == REDIS_REPLY_ERROR;
@@ -184,6 +185,9 @@ std::string to_status(redisReply &reply);
 template <typename Output>
 void to_array(redisReply &reply, Output output);
 
+template <typename Output>
+void to_optional_array(redisReply &reply, Output output);
+
 // Parse set reply to bool type
 bool parse_set_reply(redisReply &reply);
 
@@ -298,25 +302,305 @@ auto parse_tuple(redisReply **reply, std::size_t idx) ->
                             parse_tuple<Args...>(reply, idx + 1));
 }
 
-#ifdef REDIS_PLUS_PLUS_HAS_VARIANT
+template <typename T>
+bool is_parsable(redisReply &reply);
+
+bool is_parsable(ParseTag<std::string>, redisReply &reply);
+
+bool is_parsable(ParseTag<long long>, redisReply &reply);
+
+bool is_parsable(ParseTag<double>, redisReply &reply);
+
+bool is_parsable(ParseTag<bool>, redisReply &reply);
+
+bool is_parsable(ParseTag<void>, redisReply &reply);
 
 template <typename T>
-Variant<T> parse_variant(redisReply &reply) {
-    return parse<T>(reply);
+bool is_parsable(ParseTag<Optional<T>>, redisReply &reply);
+
+template <typename T, typename U>
+bool is_parsable(ParseTag<std::pair<T, U>>, redisReply &reply);
+
+template <typename ...Args>
+bool is_parsable(ParseTag<std::tuple<Args...>>, redisReply &reply);
+
+template <typename T, typename std::enable_if<IsSequenceContainer<T>::value, int>::type = 0>
+bool is_parsable(ParseTag<T>, redisReply &reply);
+
+template <typename T, typename std::enable_if<IsAssociativeContainer<T>::value, int>::type = 0>
+bool is_parsable(ParseTag<T>, redisReply &reply);
+
+#ifdef REDIS_PLUS_PLUS_HAS_VARIANT
+
+bool is_parsable(ParseTag<Monostate>, redisReply &reply);
+
+template <typename T>
+bool is_parsable(ParseTag<Variant<T>>, redisReply &reply);
+
+template <typename T, typename ...Args>
+auto is_parsable(ParseTag<Variant<T, Args...>>, redisReply &reply)
+    -> typename std::enable_if<sizeof...(Args) != 0, bool>::type;
+
+#endif
+
+template <typename T>
+inline bool is_parsable(redisReply &reply) {
+    return is_parsable(ParseTag<T>{}, reply);
+}
+
+inline bool is_parsable(ParseTag<std::string>, redisReply &reply) {
+#ifdef REDIS_PLUS_PLUS_RESP_VERSION_3
+    return is_string(reply) || is_status(reply)
+            || is_verb(reply) || is_bignum(reply);
+#else
+    return is_string(reply) || is_status(reply);
+#endif
+}
+
+inline bool is_parsable(ParseTag<long long>, redisReply &reply) {
+    return is_integer(reply);
+}
+
+inline bool is_parsable(ParseTag<double>, redisReply &reply) {
+#ifdef REDIS_PLUS_PLUS_RESP_VERSION_3
+    if (is_double(reply)) {
+        return true;
+    }
+#endif
+
+    if (!is_string(reply) || reply.str == nullptr) {
+        return false;
+    }
+
+    char *end = nullptr;
+    std::strtod(reply.str, &end);
+    return end != reply.str;
+}
+
+inline bool is_parsable(ParseTag<bool>, redisReply &reply) {
+#ifdef REDIS_PLUS_PLUS_RESP_VERSION_3
+    if (!is_bool(reply) && !is_integer(reply)) {
+        return false;
+    }
+#else
+    if (!is_integer(reply)) {
+        return false;
+    }
+#endif
+
+    return reply.integer == 0 || reply.integer == 1;
+}
+
+inline bool is_parsable(ParseTag<void>, redisReply &reply) {
+    return is_status(reply);
+}
+
+template <typename T>
+inline bool is_parsable(ParseTag<Optional<T>>, redisReply &reply) {
+    if (is_nil(reply)) {
+        return true;
+    }
+
+    return is_parsable<T>(reply);
+}
+
+template <typename T, typename U>
+inline bool is_parsable(ParseTag<std::pair<T, U>>, redisReply &reply) {
+    if (!is_array(reply)) {
+        return false;
+    }
+
+    if (reply.element == nullptr) {
+        return false;
+    }
+
+    if (reply.elements == 1) {
+        // Nested array reply. Check the first element of the nested array.
+        auto *nested_element = reply.element[0];
+        if (nested_element == nullptr) {
+            return false;
+        }
+
+        return is_parsable(ParseTag<std::pair<T, U>>{}, *nested_element);
+    }
+
+    if (reply.elements != 2) {
+        return false;
+    }
+
+    auto *first = reply.element[0];
+    auto *second = reply.element[1];
+    if (first == nullptr || second == nullptr) {
+        return false;
+    }
+
+    return is_parsable(ParseTag<typename std::decay<T>::type>{}, *first) &&
+        is_parsable(ParseTag<typename std::decay<U>::type>{}, *second);
+}
+
+template <typename T>
+bool is_tuple_parsable(redisReply **reply, std::size_t idx) {
+    assert(reply != nullptr);
+
+    auto *sub_reply = reply[idx];
+    if (sub_reply == nullptr) {
+        return false;
+    }
+
+    return is_parsable<T>(*sub_reply);
 }
 
 template <typename T, typename ...Args>
-auto parse_variant(redisReply &reply) ->
-    typename std::enable_if<sizeof...(Args) != 0, Variant<T, Args...>>::type {
+auto is_tuple_parsable(redisReply **reply, std::size_t idx) ->
+    typename std::enable_if<sizeof...(Args) != 0, bool>::type {
+    assert(reply != nullptr);
+
+    return is_tuple_parsable<T>(reply, idx) &&
+        is_tuple_parsable<Args...>(reply, idx + 1);
+}
+
+template <typename ...Args>
+bool is_parsable(ParseTag<std::tuple<Args...>>, redisReply &reply) {
+    constexpr auto size = sizeof...(Args);
+
+    static_assert(size > 0, "DO NOT support parsing tuple with 0 element");
+
+    if (!is_array(reply)) {
+        return false;
+    }
+
+    if (reply.elements != size) {
+        return false;
+    }
+
+    if (reply.element == nullptr) {
+        return false;
+    }
+
+    return is_tuple_parsable<Args...>(reply.element, 0);
+}
+
+template <typename T>
+bool is_flat_kv_parsable(redisReply &reply) {
+    if (reply.element == nullptr || reply.elements == 0) {
+        // Empty array.
+        return true;
+    }
+
+    if (reply.elements % 2 != 0) {
+        return false;
+    }
+
+    auto *key_reply = reply.element[0];
+    auto *val_reply = reply.element[1];
+    if (key_reply == nullptr || val_reply == nullptr) {
+        return false;
+    }
+
+    using Pair = typename T::value_type;
+    using FirstType = typename std::decay<typename Pair::first_type>::type;
+    using SecondType = typename std::decay<typename Pair::second_type>::type;
+    return is_parsable<FirstType>(*key_reply) &&
+        is_parsable<SecondType>(*val_reply);
+}
+
+template <typename T>
+bool is_container_parsable(std::false_type, redisReply &reply) {
+    if (reply.element == nullptr || reply.elements == 0) {
+        // Empty array.
+        return true;
+    }
+
+    auto *sub_reply = reply.element[0];
+    if (sub_reply == nullptr) {
+        return false;
+    }
+
+    return is_parsable<typename T::value_type>(*sub_reply);
+}
+
+template <typename T>
+bool is_container_parsable(std::true_type, redisReply &reply) {
+    if (is_flat_array(reply)) {
+        return is_flat_kv_parsable<T>(reply);
+    } else {
+        return is_container_parsable<T>(std::false_type{}, reply);
+    }
+}
+
+template <typename T, typename std::enable_if<IsSequenceContainer<T>::value, int>::type>
+bool is_parsable(ParseTag<T>, redisReply &reply) {
+#ifdef REDIS_PLUS_PLUS_RESP_VERSION_3
+    if (!is_array(reply) && !is_set(reply)) {
+        return false;
+#else
+    if (!is_array(reply)) {
+        return false;
+#endif
+    }
+
+    return is_container_parsable<T>(typename IsKvPair<typename T::value_type>::type(), reply);
+}
+
+template <typename T, typename std::enable_if<IsAssociativeContainer<T>::value, int>::type>
+bool is_parsable(ParseTag<T>, redisReply &reply) {
+#ifdef REDIS_PLUS_PLUS_RESP_VERSION_3
+    if (!is_array(reply) && !is_map(reply) && !is_set(reply)) {
+#else
+    if (!is_array(reply)) {
+#endif
+        return false;
+    }
+
+    return is_container_parsable<T>(std::true_type{}, reply);
+}
+
+#ifdef REDIS_PLUS_PLUS_HAS_VARIANT
+
+template <typename Result, typename T>
+Result parse_variant(redisReply &reply) {
+    if (!is_parsable<T>(reply)) {
+        throw Error("no variant type matches");
+    }
+
     auto return_var = [](auto &&arg) {
-        return Variant<T, Args...>(std::move(arg));
+        return Result(std::forward<decltype(arg)>(arg));
     };
 
-    try {
-        return std::visit(return_var, parse_variant<T>(reply));
-    } catch (const ProtoError &) {
-        return std::visit(return_var, parse_variant<Args...>(reply));
+    return std::visit(return_var, Variant<T>(parse<T>(reply)));
+}
+
+template <typename Result, typename T, typename ...Args>
+auto parse_variant(redisReply &reply) ->
+    typename std::enable_if<sizeof...(Args) != 0, Result>::type {
+    if (is_parsable<T>(reply)) {
+        auto return_var = [](auto &&arg) {
+            return Result(std::forward<decltype(arg)>(arg));
+        };
+
+        return std::visit(return_var, Variant<T>(parse<T>(reply)));
     }
+
+    return parse_variant<Result, Args...>(reply);
+}
+
+inline bool is_parsable(ParseTag<Monostate>, redisReply &) {
+    return true;
+}
+
+template <typename T>
+bool is_parsable(ParseTag<Variant<T>>, redisReply &reply) {
+    return is_parsable(ParseTag<T>{}, reply);
+}
+
+template <typename T, typename ...Args>
+auto is_parsable(ParseTag<Variant<T, Args...>>, redisReply &reply) ->
+    typename std::enable_if<sizeof...(Args) != 0, bool>::type {
+    if (is_parsable(ParseTag<T>{}, reply)) {
+        return true;
+    }
+
+    return is_parsable(ParseTag<Variant<Args...>>{}, reply);
 }
 
 #endif
@@ -414,7 +698,7 @@ std::tuple<Args...> parse(ParseTag<std::tuple<Args...>>, redisReply &reply) {
 
 template <typename ...Args>
 Variant<Args...> parse(ParseTag<Variant<Args...>>, redisReply &reply) {
-    return detail::parse_variant<Args...>(reply);
+    return detail::parse_variant<Variant<Args...>, Args...>(reply);
 }
 
 #endif
@@ -455,7 +739,7 @@ T parse(ParseTag<T>, redisReply &reply) {
 }
 
 template <typename Output>
-long long parse_scan_reply(redisReply &reply, Output output) {
+Cursor parse_scan_reply(redisReply &reply, Output output) {
     if (reply.elements != 2 || reply.element == nullptr) {
         throw ProtoError("Invalid scan reply");
     }
@@ -467,10 +751,10 @@ long long parse_scan_reply(redisReply &reply, Output output) {
     }
 
     auto cursor_str = reply::parse<std::string>(*cursor_reply);
-    long long new_cursor = 0;
+    Cursor new_cursor = 0;
     try {
-        new_cursor = std::stoll(cursor_str);
-    } catch (const std::exception &e) {
+        new_cursor = std::stoull(cursor_str);
+    } catch (const std::exception &) {
         throw ProtoError("Invalid cursor reply: " + cursor_str);
     }
 
@@ -494,6 +778,15 @@ void to_array(redisReply &reply, Output output) {
     detail::to_array(typename IsKvPairIter<Output>::type(), reply, output);
 }
 
+template <typename Output>
+void to_optional_array(redisReply &reply, Output output) {
+    if (is_nil(reply)) {
+        return;
+    }
+
+    to_array(reply, output);
+}
+
 template <typename Output>
 auto parse_xpending_reply(redisReply &reply, Output output)
     -> std::tuple<long long, OptionalString, OptionalString> {
